// ProgramResetPolarity.c
// Locations 0x20000 through 0x20003 are used to store the reset/ouput enable polarity.
// 0xff = active high reset and active low output enable.
// 0x00 = active low reset and active high output enable.
// So the memory location values determine the reset polarity.
// After programming the data polling method is used to determine the end of 
// the internal programming cycle.
// Timer/Counter1 is used to ensure the polling code exits its while loop.

#include "at17c.h"

extern volatile unsigned char t1_timed_out;
 
extern unsigned char SendByte(unsigned char byte, unsigned char order);
extern void SendStartBit(void);
extern void SendStopBit(void);
extern void ser_en( int dat );
extern void timer1_ovf0_init(void);
extern void timer1_ovf0_check(void);
 
void ProgramResetPolarity(unsigned char state) {

unsigned char i;
unsigned char test_ack = 0xff;

    ser_en( 1 ); // bring SER_EN low

    SendStartBit();
    SendByte(AT17C010 + WRITE,MSB_FIRST);  // send device address byte
    SendByte(0x02,MSB_FIRST);                 // 1st address byte
    SendByte(0x00,MSB_FIRST);                 // 2nd address byte
    SendByte(0x00,MSB_FIRST);                 // 3rd address byte ... most significant byte
 
    for (i = 0; i < 4; i++)
        SendByte(state,LSB_FIRST);
            
    SendStopBit();

    timer1_ovf0_init(); // set in timer counter 0 overflow interrupt routine
 
    // 20 milli-second timeout
    // 7.3728MHz / 1024 = 7200 Hz
    // 7200 Hz = 138.8 us
    // 20 ms / 138.8 us = 144.09
    // 65536 - 144 = 65392 = ff70
    // interrupt on ffff to 0000 transition
    //TCNT1H = 0xff;  
    //TCNT1L = 0x70; // load counter
    //TCCR1B = 0x05; // timer/counter 1 clock / 1024

    // continue sending start bit and device address until we get an ack back
    // data poll to program complete ... time out for error
    while (test_ack && !t1_timed_out) {
	timer1_ovf0_check();
        SendStartBit();
        test_ack = SendByte(AT17C010 + WRITE,MSB_FIRST); // send device address byte
    }
    SendStopBit();

    ser_en( 0 ); // bring SER_EN high
        
}
