/*----------------------------------------------------------------------
io.c

Linux Base 

SDA - bidirectional
Output = D0 = 0/1 = Low_Output/High_Output(Input-Mode)
Input  = ACK

SCL - output
Output = D7 = 0/1 = Low/High

SER_EN - output
Nothing
----------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/io.h>
#include <asm/ioctl.h>

#include "at17c.h"


extern volatile unsigned char t0_timed_out;
extern volatile unsigned char t1_timed_out;


#define IO_WAIT ( 0x80 )
#define IO_WAIT_SIZE ( 1 )

//#define IO_PORT ( 0x278 )
#define IO_PORT ( 0x378 )
#define IO_PORT_SIZE ( 8 )
#define IO_PORT_DATA ( IO_PORT + 0 )	// ǡBUS D7..0
#define IO_PORT_STAT ( IO_PORT + 1 )	// /BUSY /ACK PE SLCT /Error NA NA NA
#define IO_PORT_CTRL ( IO_PORT + 2 )	// NA NA NA IRQ_En SLCT /INIT AUTO_FD_XT STROBE_Pulse

 
void Init( void )
{
    if ( ioperm( IO_WAIT, IO_WAIT_SIZE, 1 ) != 0 ) {
	fprintf( stderr, "\nCannot Access I/O Ports.\n" );
	exit( 1 );
    }


    if ( ioperm( IO_PORT, IO_PORT_SIZE, 1 ) != 0 ) {
	fprintf( stderr, "\nCannot Access I/O Ports.\n" );
	exit( 1 );
    }
    outb( 0x0a, IO_PORT_CTRL );
    outb( 0x01, IO_PORT_DATA );
}


void Close( void )
{
    ioperm( IO_PORT, IO_PORT_SIZE, 0 );
    ioperm( IO_WAIT, IO_WAIT_SIZE, 0 );
}


void ser_en( int dat )
{
    /* Not Assign (dummy) */
}


void set_sda_in( void )
{
    outb( inb( IO_PORT_DATA ) | 0x01, IO_PORT_DATA );
}

unsigned char read_sda( void )
{
    return ( inb( IO_PORT_STAT ) & 0x40 ) ? 1 : 0;
}


unsigned char read_scl( void )
{
    return ( inb( IO_PORT_DATA ) & 0x80 ) ? 1 : 0;
}


void WriteSCL( unsigned char state )
{
    unsigned char dat;

    dat = inb( IO_PORT_DATA );
    if ( state ) {
	outb( dat | 0x80, IO_PORT_DATA );
    } else {
	outb( dat & ~0x80, IO_PORT_DATA );
    }
}


void WriteSDA( unsigned char state )
{
    unsigned char dat;

    dat = inb( IO_PORT_DATA );
    if ( state ) {
	outb( dat | 0x01, IO_PORT_DATA );
    } else {
	outb( dat & ~0x01, IO_PORT_DATA );
    }
}


void BitDelay(void)
{
    int cnt;

    // wait 4us
    for ( cnt = 0; cnt < 4; cnt ++ ) {
	WAIT_1US;
    }
    
    return;
}


unsigned short timer0;
unsigned short timer1;

void timer0_ovf0_init(void)
{
    timer0 = 0;
    t0_timed_out = FALSE;
}


void timer0_ovf0_check(void)
{
    WAIT_1US;
    timer0 ++;

    if ( timer0 > 50 ) {
	t0_timed_out = TRUE;
    }
}

void timer1_ovf0_init(void)
{
    timer1 = 0;
    t1_timed_out = FALSE;
}


void timer1_ovf0_check(void)
{
    WAIT_1US;
    timer1 ++;

    if ( timer1 > 20000 ) {
	t1_timed_out = TRUE;
    }
}


unsigned char SetSCLHigh(void)
{
    WriteSCL(1); // release SCL

    // set up timer counter 0 for timeout
    timer0_ovf0_init();

    // wait till SCL goes to a 1
    while (!read_scl() && !t0_timed_out) {
	timer0_ovf0_check();
    }

    return(t0_timed_out);
}
