// main
// use the atstk200 starter kit and external prototype board to program
// an Atmel AT17C010 device. Leds on PORTC are used for visual status indicators
// 4.7k ohm pull-up resistors are used on SDA and SCL


#define FILEOUT
#define SFN "xilinx.bst"
#define DFN "xilinx.bin"


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "at17c.h"

volatile unsigned char t0_timed_out;
volatile unsigned char t1_timed_out;
unsigned char wrbuf[PAGE_SIZE];
unsigned char rdbuf[PAGE_SIZE];
unsigned char ram[ PAGE_SIZE * MAX_PAGES ];

extern void WritePage(unsigned int address, unsigned char *bufptr);
extern void ReadPage(unsigned int address, unsigned char *bufptr);
void ProgramResetPolarity(unsigned char state);
unsigned char VerifyResetPolarity(void);
extern void Init(void);
extern void Close( void );


int debug_dump( char *dat, int size )
{
    char disp_str[MAXSTR];
    char tmp[MAXSTR];
    int add;
    int cnt;

    for ( add = 0; add < size; add += DUMP_L_SIZE ) {
	sprintf( disp_str, "%08X ", add );
	for ( cnt = 0; cnt < DUMP_L_SIZE; cnt++ ) {
	    sprintf( tmp, " %02X", ( unsigned char )dat[ add + cnt ] );
	    strcat( disp_str, tmp );
	}
	strcat( disp_str, "  " );
	for ( cnt = 0; cnt < DUMP_L_SIZE; cnt++ ) {
	    if ( isprint( dat[ add + cnt ] ) == 0 ) {
		strcat( disp_str, "." );
	    }
	    else {
		sprintf( tmp, "%c", dat[ add + cnt ] );
		strcat( disp_str, tmp );
	    }
	}
	printf( "%s\n", disp_str );
    }

    return 0;
}


unsigned char asc2bin( char *buf )
{
    unsigned char dat;
    int cnt;

    dat = 0;
    for ( cnt = 7; cnt >= 0; cnt -- ) {
	if ( buf[ 7 - cnt ] == '1' ) {
	    dat += 1<<cnt;
	}
    }

    return dat;
}


int set_ram( void )
{
    FILE   *rd_fp;
#ifdef FILEOUT
    FILE   *wr_fp;
#endif
    char fname[ MAXSTR ];
    char buffer[ MAXSTR ];
    unsigned long add;
    int cnt;

    printf( "Start convert bst-file[%s]", SFN );
#ifdef FILEOUT
    printf( " -> bin-file[%s]", DFN );
#endif
    putchar( '\n' );

    // Clear ram
    for ( add = 0; add < PAGE_SIZE * MAX_PAGES; ) {
	for ( cnt = 0; cnt < PAGE_SIZE; cnt ++ ) {
	    ram[ add ] = 0x00;
	    add ++;
	}
	putchar( '.' );
    }
    printd( "\nClear ram memory.\n" );

    // Open bst-file
    strcpy( fname, SFN );
    if ( ( rd_fp = fopen( fname, "r" ) ) == NULL ) {
	printf( "Can't Open file [%s]!\n", fname );
	return -1;
    }
    printd( "Open file.\n" );

#ifdef FILEOUT
    strcpy( fname, DFN );
    if ( ( wr_fp = fopen( fname, "wb" ) ) == NULL ) {
	printf( "Can't Open file [%s]!\n", fname );
	return -1;
    }
#endif

    /* 1ɤ߹ǤϥХʥѴ */
    add = 0;
    while ( fgets( buffer, MAXSTR, rd_fp ) != NULL ) {
	ram[ add ] = asc2bin( buffer );
	//buffer[ 8 ] = '\0';
	//printf(" buf = %s, dat = %02XH\n", buffer, ram[ add ] );
#ifdef FILEOUT
	fputc( (int)ram[ add ], wr_fp );
#endif
	add ++;
    }
    printd( "Set bin-buf finish.\n" );

    // Close bst-file
#ifdef FILEOUT
    fclose( wr_fp );
#endif
    fclose( rd_fp );
    printd( "Close file.\n" );

    printf( "Finish.\n" );
    return 0;
}


int main( int argc, char *argv[] )
{
    unsigned int address = 0;
    unsigned char i;

    Init();

    _SEI(); // enable interrupts

    // init ram buffer
    set_ram();
 
    for (address = 0; address < MAX_PAGES; address++) {
	// load wrbuf
	for (i = 0; i < PAGE_SIZE; i++) {
	    wrbuf[i] = ram[ (unsigned long)address * PAGE_SIZE + i ];
	}

	WritePage(address,&wrbuf[0]);
	printf( "%06X\n", address );
	ReadPage(address,&rdbuf[0]);

#ifdef DEBUG
	puts( "wrbuf" );
	debug_dump( wrbuf, PAGE_SIZE );
	puts( "rdbuf" );
	debug_dump( rdbuf, PAGE_SIZE );
#endif
            
	// verify programmed page
	for (i = 0; i < PAGE_SIZE; i++) {
	    if (wrbuf[i] != rdbuf[i]) {
		puts( "Verify NG" );
	    }    
	}
            
	// clear verify buf
	for (i = 0; i < PAGE_SIZE; i++)
	    rdbuf[i] = 0; 
    }    

    ProgramResetPolarity( 0xff );
    if ( VerifyResetPolarity() != 0xff ) {
	puts( "VerifyResetPolarity( 0xff ) NG" );
    }    

    Close();

    return 0;
}
